@@ -15,17 +15,21 @@ import cloudpathlib
 import pandas as pd
 import pytest
 
+from pynock import Partition
+
 
 def test_pandas ():
     df_csv = pd.read_csv(
         cloudpathlib.AnyPath("dat/tiny.csv"),
-    ).fillna("")
+    ).fillna("").sort_values(Partition.SORT_COLUMNS).reset_index(drop=True)
+
     ic(df_csv.iloc[:, [2, 3, 7]])
 
     df_parq = pd.read_parquet(
         cloudpathlib.AnyPath("dat/tiny.parq"),
         use_nullable_dtypes = True,
-    ).fillna("")
+    ).fillna("").sort_values(Partition.SORT_COLUMNS).reset_index(drop=True)
+
     ic(df_parq.iloc[:, [2, 3, 7]])
 
     # general diff
